package BCD::Bcd;

use strict;
use XML::Simple;
use Data::Dumper;
use Pod::Usage;

use BCD::Common qw(:DEFAULT $isoconf $wd $name $arch $version $isodir $builddir $error_color);
use BCD::Genisoimage;
use BCD::Isolinux;
use BCD::Stagex;
use BCD::Media;

our @ISA = qw(Exporter);
our @EXPORT = qw(main_bcd);
my $color = "cyan";

sub print_info() {
	print Dumper($isoconf);

}

sub help_bcd {
	pod2usage('-verbose' => 2, '-input' => "bcd.pod", '-pathlist' => @INC);
}

sub do_all {
	my ($option) = @_;
	clean_all;
	check_dir;
	main_isolinux;
	main_stagex;
	main_media($option);
	main_iso($option);
	show_info();
}

sub show_info {
     print "\n";
     print "ISO: $isodir\n";
     print "build dir: $builddir\n";
     print "LOG of media: /tmp/$name-$version\n";
     print "\n";
}

sub main_bcd() {
    my %opts = (
	'' => \&help_bcd,
	info => \&print_info,
	iso => \&main_iso,
	md5 => \&create_md5,
	stagex => \&main_stagex,
	isolinux => \&main_isolinux,
	media => \&main_media,
	gendistrib => \&use_gendistrib,
	clean => \&clean_all,
	list => \&list_media,
	mediarepo => \&list_medias_available,
	checkrepo => \&check_repo_hdlist,
	all => \&do_all,
	);
    
    $ARGV[0] or help_bcd;
    -f $ARGV[0] or die "Cant acces to conf file $ARGV[0]\n";
    if (my $f = $opts{$ARGV[1]}) {
	if ($ARGV[2]) {
		my $options;
		foreach my $arg (2 .. $#ARGV) {
		    $arg and $options = $options . ' ' . $ARGV[$arg];
	    	}
		$f->($options);
	} else {
		$f->();
	}
    } else {
	print_color(" ** Dont know what todo ** \n unknow option", $error_color);
    }
}


1;
