package BCD::Common;

use strict;
use XML::Simple;
use MDK::Common;
use Term::ANSIColor;
use File::Copy::Recursive qw(dircopy pathrm);
use File::Glob ':glob';

use BCD::Bcd qw(:DEFAULT $conf_file);

our @ISA = qw(Exporter);
our @EXPORT = qw(create_needed_dir check_dir clean_all print_color);
# export all needed var
our @EXPORT_OK = qw($isoconf $wd $name $arch $file $based_on $version $repo $isodir $builddir $distrib_path $error_color $DISTRIB);

my $LOG="COMMON -";
my $color = "red";


my $conf_file = $ARGV[0];
our $isoconf = XMLin($conf_file, keyattr => ['']);

our $DISTRIB = $isoconf->{theme}{name};
our $name = $isoconf->{nameid};
our $wd = $isoconf->{workdir}{path};
our $arch = $isoconf->{arch};
our $version = $isoconf->{mediainfo}{version};
our $repo = $isoconf->{repo}{path};
our $based_on = $isoconf->{based_on};
our $builddir = "$wd/$isoconf->{iso}{genisoimage}{builddir}/$name-$version-$arch/$arch";
our $isodir = "$wd/$isoconf->{iso}{genisoimage}{isodir}/$name-$version-$arch";
# distrib path in chroot (mounted loop)
our $distrib_path = "/tmp/distrib";
our $error_color = "blue on_magenta";

sub print_color {
	my ($text, $color) = @_;
	print color $color;
	print "$text\n";
	print color 'reset';
	
}

sub create_needed_dir {
	my ($dir) = @_;
        # create needed dir to create the build tree, and to store the ISO
	if (! -d $dir) {
		#mkdir $dir, 0755;
		print_color("$LOG creation of $dir", $color);
		mkdir_p($dir);
	} else {
		print_color("$LOG $dir already exist", $color);
	}
}

sub check_dir {
	create_needed_dir($builddir);
	create_needed_dir("$builddir/$isoconf->{mediainfo}{mediainfo_dir}");
	create_needed_dir($isodir);
	create_needed_dir("/tmp/$isoconf->{nameid}-$version-$isoconf->{arch}");
}

sub clean_all {
	print_color("$LOG rmdir $builddir", $color);
	#pathrm("$wd/$isoconf->{iso}{genisoimage}{builddir}/$name/$arch") or die $!;
        -d $builddir and system("rm -rf $builddir");
	print_color("$LOG rmdir $isodir", $color);
	#pathrm("$wd/$isoconf->{iso}{genisoimage}{isodir}/$name/$arch") or die $!;
	-d $isodir and system("rm -rf $isodir");
	my @l = glob("/tmp/$isoconf->{nameid}-$version-$isoconf->{arch}/$distrib_path/*");
	foreach (@l) {
	    $_ or next;
	    if (-f "$_" || -d "$_") { 
		print_color("Distribution repositery already mounted exiting !\n Check /tmp/$isoconf->{nameid}-$version-$isoconf->{arch}/$distrib_path directory", $color);
		print_color("sudo umount /tmp/$isoconf->{nameid}-$version-$isoconf->{arch}$distrib_path", $error_color);
		foreach my $media (@{$isoconf->{media}{list}}) {
		    $media->{name} or next;
		    $media->{external}{name} and print_color("You should check also umount the external media /tmp/$isoconf->{nameid}-$version-$isoconf->{arch}/tmp/$media->{external}{name}\nsudo umount /tmp/$isoconf->{nameid}-$version-$isoconf->{arch}/tmp/$media->{external}{name}", $error_color);
		}
		print_color("WARNING: if you didnt umount repositery or external media you can erase them unless they are mounted bind in RO mode.", $color);
		exit 1;
		}
	}
	print_color("$LOG rmdir /tmp/$isoconf->{nameid}-$version-$isoconf->{arch}", $color);
	system("sudo rm -rf /tmp/$isoconf->{nameid}-$version-$isoconf->{arch}");
}
1;
