package BCD::Media;

use File::Copy::Recursive qw(dircopy pathrm);
use strict;
use URPM;
use File::Copy;
use File::stat;
use Data::Dumper;
use MDK::Common;

use BCD::Common qw(:DEFAULT $isoconf $wd $name $arch $version $based_on $repo $builddir $distrib_path $error_color $DISTRIB);

our @ISA = qw(Exporter);
our @EXPORT = qw(main_media list_media use_gendistrib list_medias_available check_repo_hdlist);

my $LOG="MEDIA -";
my $color = "magenta";

# all mounted loop dir to umount
my @to_umount;
my @add_media;

my $mediainfo_sourcedir;
if (defined($isoconf->{mediainfo}{fullpath})) {
	$mediainfo_sourcedir = $isoconf->{mediainfo}{fullpath};
} else {
	$mediainfo_sourcedir = "$repo/$based_on/$arch/$isoconf->{mediainfo}{mediainfo_dir}";
}
my $chroot_path = "/tmp/$isoconf->{nameid}-$version-$isoconf->{arch}";

sub create_media {
    print_color("$LOG Create the media dir in $builddir", $color);
    foreach my $media (@{$isoconf->{media}{list}}) {
	create_needed_dir("$builddir/media/$media->{destmedia}");
    }
}

sub show_size {
	print_color("$LOG size of the builddir", $color);
	system("du -sLh $builddir");
}

sub list_media {
    print_color("$LOG all media are listed there:", $color);
    foreach my $media (@{$isoconf->{media}{list}}) {
    $media->{name} or next;
    print "
name: $media->{name}
    desc: $media->{desc}
    mediadir: $media->{mediadir}
    destmedia: $media->{destmedia}
    pubkey: $media->{pubkey}\n";
	foreach my $files (@{$media->{input}}) {
		$files->{file} or next;
		if (-f $files->{file}) {
			print "list of wanted: $files->{file}\n";
		} else { print_color("$files->{file} not present", $error_color); }
 	}
	foreach my $files (@{$media->{exclude}}) {
		$files->{file} or next;
		if (-f $files->{file}) {
			print "list of exclusion: $files->{file}\n";
		} else { print_color("$files->{file} not present", $error_color); }
 	}
   }
}

sub copy_pubkey {
    my ($source_key, $destmedia) = @_;
    print_color("$LOG copy $source_key to the $builddir/media/$destmedia/media_info directory", $color);
    create_needed_dir("$builddir/media/$destmedia/media_info/");
    if (! -f $source_key) {
	print_color("$LOG can't find the $source_key", $error_color);
	#exit;
    }
    system("cp -vf $source_key $builddir/media/$destmedia/media_info/");
}

sub use_gendistrib {
	my $destmedia;
    	print_color("$LOG remove hdlists and synthesys from $builddir/media/$destmedia/media_info and $builddir/media/media_info", $color);
	system("rm -vf $builddir/media/media_info/*.cz");
	foreach my $media (@{$isoconf->{media}{list}}) {
	    $media->{name} or next;
	    if ($destmedia eq $media->{destmedia}) { next }
	    $destmedia = $media->{destmedia};
	    system("rm -vf $builddir/media/$destmedia/media_info/*.cz");
	    system("rm -vf $builddir/media/$destmedia/media_info/MD5SUM");
	}
	undef $destmedia;
    	print_color("$LOG gendistrib --mediacfg $builddir/$isoconf->{iso}{mediacfg} $builddir", $color);
    	system("gendistrib --mediacfg $builddir/$isoconf->{iso}{mediacfg} $builddir");
	if ($isoconf->{based_on} lt "2009.1") {
		foreach my $media (@{$isoconf->{media}{list}}) {
		$media->{name} or next;
		if ($destmedia eq $media->{destmedia}) { next }
		$destmedia = $media->{destmedia};
		print_color("$LOG move hdlists and synthesys from $builddir/media/$destmedia/media_info to $builddir/media/media_info/", $color);
		system("rm -vf $builddir/media/media_info/*_$media->{name}.cz");
		system("rm -vf $builddir/media/media_info/pubkey_$media->{name}");
		system("cp -avf $builddir/media/$destmedia/media_info/hdlist.cz $builddir/media/media_info/hdlist_$media->{name}.cz");
		system("cp -avf $builddir/media/$destmedia/media_info/pubkey $builddir/media/media_info/pubkey_$media->{name}");
		system("cp -avf $builddir/media/$destmedia/media_info/synthesis.hdlist.cz $builddir/media/media_info/synthesis.hdlist_$media->{name}.cz");
		system("rm -vf $builddir/media/$destmedia/media_info/hdlist*.cz");
		system("rm -vf $builddir/media/$destmedia/media_info/MD5SUM");
		} 
	}
}

sub copy_rpm {
	my ($source_dir, $dest_dir) = @_;
	system ("cp -a $source_dir/*.rpm $dest_dir"); 
}

sub create_mediacfg {
    print_color("$LOG $builddir/$isoconf->{iso}{mediacfg}", $color);
    system("rm -rf $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
    create_needed_dir("$builddir/$isoconf->{mediainfo}{mediainfo_dir}");
    my $media_cfg = "$builddir/$isoconf->{iso}{mediacfg}";
    open my $MEDIACFG, ">$media_cfg";
    print $MEDIACFG "\n";
    print $MEDIACFG "[media_info]\n";
    print $MEDIACFG "version=$version\n";
    if ($isoconf->{based_on} gt "2009.1") {
    	print $MEDIACFG "mediacfg_version=2\n";
    }
    print $MEDIACFG "branch=$isoconf->{iso}{branch}\n";
    print $MEDIACFG "arch=$arch\n";
    print $MEDIACFG "minor=$isoconf->{iso}{minor}\n";
    print $MEDIACFG "subversion=$isoconf->{iso}{subversion}\n";
    if ($isoconf->{mediainfo}{suppl}{todo} eq "yes") {
        print $MEDIACFG "suppl=1\n";
    }
    if ($isoconf->{mediainfo}{askmedia}{todo} eq "yes") {
	print $MEDIACFG "askmedia=1\n";
    }
    if ($isoconf->{mediainfo}{xmlinfo}{todo} eq "yes") {
	print $MEDIACFG "xml-info=1\n";
    }
    my $destmedia;
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	if ($destmedia eq $media->{destmedia}) { next }
	$destmedia = $media->{destmedia};
	if ($isoconf->{based_on} lt "2009.1") {
	print $MEDIACFG "

[$media->{destmedia}]
hdlist=hdlist_$media->{name}.cz
pubkey=pubkey_$media->{name}
name=$isoconf->{iso}{header}{volumesetid}
desc=$isoconf->{iso}{header}{volumesetid}
\n";
} else {
	print $MEDIACFG "
[$media->{destmedia}]
synthesis=$media->{destmedia}/media_info/synthesis.hdlist.cz
hdlist=$media->{destmedia}/media_info/hdlist.cz
pubkey=$media->{destmedia}/media_info/pubkey
name=$media->{desc}
\n";
    	}
    }
    close $MEDIACFG;
}

# based on a similar functions in pkgs.pm
sub get_list_from_CAT {
	#4 META_CLASS"desktop" mandriva-xfce-config-One
    my ($file, $cat, $poids) = @_;
    my (%rates, @need_to_copy);
    my (@l);
    local $_;
    foreach (cat_($file)) {
        s/#.*//; # comments
        my ($indent, $data) = /(\s*)(.*)/;
        next if !$data; # skip empty lines
	
        @l = grep { $_->[0] < length $indent } @l;

        my @m = @l ? @{$l[-1][1]} : ();
        my ($t, $flag, @l2);
        while ($data =~
               /^((
                   [1-5]
                   |
                   (?:            (?: !\s*)? [0-9A-Z_]+(?:".*?")?)
                   (?: \s*\|\|\s* (?: !\s*)? [0-9A-Z_]+(?:".*?")?)*
                  )
                  (?:\s+|$)
                 )(.*)/x) { #@")) {
            ($t, $flag, $data) = ($1,$2,$3);
            while ($flag =~ s,^\s*(("[^"]*"|[^"\s]*)*)\s+,$1,) {}
	    push @m, $flag;
	    push @l2, [ length $indent, [ @m ] ];
	    $indent .= $t;
        }
        if ($data) {
            # has packages on same line
            my ($rates, $flags) = partition { /^\d$/ } @m;
            foreach my $name (split ' ', $data) {
                if (member($cat, @$flags)) {
		    if ("@$rates" ge "$poids") {
			push @need_to_copy, $name;
		    } else {
			print "exclude $name of $cat (@$rates < $poids)\n";
		    }
                    next;
                }
            }
            push @l, @l2;
        } else {
            push @l, [ $l2[0][0], $l2[-1][1] ];
        }
    }
    \%rates, \@need_to_copy;
}

sub mini_chroot {
	# it is usefull to use a mini chroot for add remove media, we have to keep the list of installed packages to copy them on ISO
	print_color("$LOG create a mini chroot to run urpmi", $color);
	# store all file needed to be installed on the system
	my $cmd = "sudo urpmi --no-suggests --quiet --auto --no-verify-rpm --use-distrib $repo/$based_on/$arch --urpmi-root $chroot_path";
	#  sudo urpmi --no-verify-rpm --auto --urpmi-root /tmp/mk --use-distrib /mnt/BIG/dis/2009.0/i586/ basesystem-minimal urpmi --test
	#
	system("sudo urpmi --test --no-verify-rpm --auto --use-distrib $repo/$based_on/$arch --urpmi-root /tmp/$isoconf->{nameid}-$version-$isoconf->{arch} urpmi basesystem-minimal bash basesystem > $chroot_path/$isoconf->{tocopy_file}_todo_chroot");
	# install all those file to be able to use urpmi.XXXXXX
	system("$cmd urpmi bash --prefer $DISTRIB");
        system("sudo chroot $chroot_path mkdir -p $distrib_path");
	system("sudo mount -o bind $repo/$based_on/$arch $chroot_path/$distrib_path -o ro");
	push @to_umount, $distrib_path;
	system("sudo chroot $chroot_path urpmi.removemedia -a");
	foreach my $media (@{$isoconf->{media}{list}}) {
		$media->{name} or next;
		chomp($media->{name});
	    	system("sudo chroot $chroot_path urpmi.addmedia $media->{name} $distrib_path/media/$media->{mediadir}/release");
		if ($media->{updates} eq "yes") {
			system("sudo chroot $chroot_path urpmi.addmedia '$media->{name} Updates' $distrib_path/media/$media->{mediadir}/updates");
		}
		if ($media->{backports} eq "yes") {
			system("sudo chroot $chroot_path urpmi.addmedia '$media->{name} Backports' $distrib_path/media/$media->{mediadir}/updates");
		}
		if ($media->{testing} eq "yes") {
			system("sudo chroot $chroot_path urpmi.addmedia '$media->{name} Testing' $distrib_path/media/$media->{mediadir}/updates");
		}
	}
#        } else {
#            system("sudo chroot $chroot_path urpmi.addmedia --distrib $distrib_path --no-md5sum");
#	}
}

sub add_media {
	my ($name, $path) = @_;
	print_color("$LOG add media $name from $path", $color);
	system("sudo chroot $chroot_path mkdir -p /tmp/$name");
	system("sudo mount -o bind $path $chroot_path/tmp/$name -o ro");
        system("sudo chroot $chroot_path urpmi.addmedia $name /tmp/$name");
	push @to_umount, "/tmp/$name";
	push @add_media, "$name";
}


my $listcopy = "$chroot_path/$isoconf->{tocopy_file}";
sub generate_list_of_rpm_2_copy {
    create_needed_dir($chroot_path);
    my @toget;my @toexclude;my $extra;
    mini_chroot;
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	$media->{external}{name} and add_media($media->{external}{name}, $media->{external}{fullpath_mediadir});
	foreach my $input (@{$media->{input}}) {
	    $input->{file} or next;
	    -f $input->{file} or print_color("$LOG i can not find the $input->{file}, remove this list or fix...", $error_color);
	    foreach my $tohave (cat_($input->{file})) {
		if ($tohave =~ /^\s/ || $tohave =~ /^\#/) {
		    next;
		} elsif ($tohave =~ /^\S*\s\d$/) {
		    my ($section, $poids) = $tohave =~ /^(\S*)\s(\d)/;
		    print "$LOG get packages from $section using weight of $poids\n";
		    my $tmp_rpmsrate = "/tmp/rpmtemp";
		    # TOFIX remove META_CLASS to avoid theme pb
		    system("cat $builddir/$isoconf->{mediainfo}{mediainfo_dir}/rpmsrate | grep -v META_CLASS > $tmp_rpmsrate");
	            my ($rates, $need_to_copy) = get_list_from_CAT($tmp_rpmsrate, $section, $poids);
		    foreach my $data (@{$need_to_copy}) {
			push @toget, $data;
		    }
		    next;
		} else {
		    chomp($tohave);
		    push @toget, $tohave;
		    next;
		}
	    }
	}
	foreach my $exclude (@{$media->{exclude}}) {
	    $exclude->{file} or next;
	    -f $exclude->{file} or print_color("$LOG i can not find the $exclude->{file}, remove this list or fix...", $error_color);
	    foreach my $totest (cat_($exclude->{file})) {
		if ($totest =~ /^\s/ || $totest =~ /^\#/) {
		    next;
		} else {
		    chomp($totest);
		    push @toexclude, $totest;
		    next;
		}
	    }
	}
#	print "$LOG $media, $toget, $toexclude";
    	create_list($media, \@toget, \@toexclude, $media->{external}{name}, $media->{media_supp});
	@toexclude = "";
	@toget = "";
    }
}

sub create_list {
    my ($media, $wanted, $exclude, $extra, $media_supp) = @_;
    my $extra_media;
    # Main media is mandatory for all other media !
    # we will use only Main + updates and this media + updates to calculte the list of wanted pkg
    # Updates should be a parameter (later)

	if ($media->{name} eq "Main") {
	$extra_media = "--media \"Main,Main Updates,$extra,$media_supp\" --sortmedia \"Main Updates,Main\"";
	} else {
	my $extra2;
	foreach (@add_media) { chomp($_); $_ and $extra2 = $_ . ',' . $extra2; };
	$extra_media = "--media \"Main,Main Updates,$extra2,$media_supp,$media->{name},$media->{name} Updates\" --sortmedia \"Main Updates,Main,$media->{name} Updates,$media->{name}\"";
	}
    my $opt_sug;
    # check the suggests option in the XML configuration
    if ($media->{nosuggests} eq "yes") {
	$opt_sug = "--no-suggests";
    }
    # list of wanted RPM per media name
    my $list = "$chroot_path/$isoconf->{tocopy_file}_todo_$media->{name}";
    # urpmi error file
    my $urpmi_error = "$chroot_path/$isoconf->{tocopy_file}_urpmi_error_$media->{name}";

    # remove main32 and main32 updates for x86_64 iso
#    my $exclude_media;
#    if ($arch eq "x86_64") {
#	   $exclude_media = "--excludemedia 'Main32,Main32 Updates'";
#   }
    my $tested_wanted;
    my $tested_exclude;
    foreach (@{$wanted}) { $tested_wanted = $tested_wanted . " $_" }
    foreach (@{$exclude}) { $tested_exclude = $tested_exclude . " --skip $_"  };
    print_color("$LOG remove unwanted mandriva-release, use $DISTRIB", $color);
    # urpmi optipns
    my $urpmi_options = "--no-md5sum --no-verify-rpm --ignorearch --auto --test --nolock $isoconf->{mediainfo}{urpmi_option}";
    my $urpmi_extra_opt = "$opt_sug $extra_media";
    my $cmd = "sudo chroot $chroot_path urpmi --prefer $DISTRIB $urpmi_options $urpmi_extra_opt $tested_wanted $tested_exclude > $list 2> $urpmi_error";
    print_color("$LOG first attempt to run urpmi on media $media->{name} (to find all packages not available)", $color);
    system($cmd);
    # urpmi error found, so we have to remove some entry in the list of wanted packages
    my @toremove;
    if (-f $urpmi_error) {
	foreach my $entry (cat_($urpmi_error)) {
            my ($data, $tor) = $entry =~ /(.*)\s(.*)\n/;
            $data =~ /-/ and next;
	    $tor or next;
	    push @toremove, $tor;
	}
	my %already_seen; my @final_list;
	@already_seen{@toremove} = ();
	foreach my $object (@{$wanted}) {
	    if (exists $already_seen{$object}) { print_color("$LOG i cant find this package: $object, remove it from list", $error_color); }
	    push(@final_list, $object) unless exists $already_seen{$object};
	}
	my $final_list;
	foreach (@final_list) {
		chomp($_);
		$final_list = $final_list . " $_";
	}
        my $cmd2 = "sudo chroot $chroot_path urpmi --prefer $DISTRIB $urpmi_options $urpmi_extra_opt $final_list $tested_exclude > $list";
        print_color("$LOG $cmd2", $color);
	system($cmd2);
    }
#    if ($media->{name} eq "Main") { print "No extra from Media $media->{name}\n" } else { remove_extra_from_main($list, $wanted, $media); }
}

# TODO
# remove unwanted RPM taken from main, while wanted from other media
sub remove_extra_from_main {
	my ($list, $wanted, $media) = @_;
	# remove package in main from $wanted in $list (wrongly added)
#for i in `cat ~/bcd/lists/input_contrib` ; do grep $i tocopy_plop_todo_Contrib | grep "\/main/.*/$i" ; done
	
	my $list_to_remove = "";
	# remove deps in main for those RPM
	my $urpmi_options = "--no-verify-rpm --auto --ignoresize --test --nolock";
#	my $cmd = "sudo urpmi $opt_sug $urpmi_options $extra_media --use-distrib $repo/$based_on/$arch/ --urpmi-root $chroot_path/ $list_to_remove > $list.deps_to_remove";
#	print "\n $LOG $cmd \n";
	# remove all those RPM from the $list
	# $list.deps_to_remove and list_to_remove
}

sub concatenate_files {
    print_color("$LOG create the final list: $listcopy", $color);
    system("cat $chroot_path/$isoconf->{tocopy_file}_todo_* > $listcopy.tmp");
    system("cat $listcopy.tmp | sort | uniq > $listcopy");
}

sub target_size {
	print_color("$LOG calculate the size of all packages:", $color);
	my $total;my $pkg_info;
	foreach my $pkg (cat_($listcopy)) {
		chomp($pkg);
		if (-f "$pkg") {
		# stat is not really good to get size file of linked packages over nfs
		$pkg_info = stat($pkg);
		$total = ($total+$pkg_info->size);
		next;
		} else { next; }
	}
	my $human_total = int(($total/1024)/1024);
	print_color("$LOG Size of all RPMS: $human_total Mo", $color);
	my $left = ($isoconf->{iso}{size} - $human_total);
	# i need to add the install/ directory size
	if ("$isoconf->{iso}{size}" ge "$human_total") {
		print_color("$LOG all you data will fit on the CD/DVD, extra size: $left", $color);
	} else {
		print_color("$LOG all packages will not fit on $isoconf->{iso}{size}, i need $left Mo", $error_color);
	}
}

sub count_nb_to_copy {
    my ($file) = @_;
    my $lines = 0;
    open(FILE, $file) or die "Can't open `$file': $!";
    while (sysread FILE, my $buffer, 4096) {
	if ($buffer =~ /^install/ || $buffer =~ /^\-/ || $buffer =~ /^More/ || $buffer =~ /^Install/ ) {
	    next;
	} else {
	    $lines += ($buffer =~ tr/\n//);
	}
    }
    close FILE;
    return $lines;
}

sub find_source_dest_media {
    my ($todeal) = @_;
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	#print "--------\n todeal: $todeal\n";
	#print "find_source_dest_media\n media->{name}: $media->{name}\n fullpath_mediadir: $media->{external}{fullpath_mediadir}\n destmedia: $builddir/media/$media->{destmedia}\n";
	# external media
	if ($todeal =~ m!/tmp/$media->{name}!) {
		#print "fullpath_mediadir: $media->{external}{fullpath_mediadir} destmedia: $builddir/media/$media->{destmedia}\n";
		return($media->{external}{fullpath_mediadir}, "media/$media->{destmedia}");
	# distrib media
	} elsif ($todeal =~ m!$distrib_path!) {
                if ($todeal =~ m!$media->{mediadir}!) {
			my ($f_path) = $todeal =~ m!$media->{mediadir}/(.*)!;
#			print "find_source_dest_media RETURN $repo/$based_on/$arch/media/$media->{mediadir}/$f_path AND media/$media->{destmedia}\n";
#			print "find_source_dest_media media->{name}: $media->{name} RETURN fullpath_mediadir: $media->{external}{fullpath_mediadir} TO destmedia: $builddir/media/$media->{destmedia}\n";
			return("$repo/$based_on/$arch/media/$media->{mediadir}/$f_path", "media/$media->{destmedia}");
			next;
               } else { 
		       print "lost in space2 !\ntodeal = $todeal\n";
		       #print "find_source_dest_media\n media->{name}: $media->{name}\n fullpath_mediadir: $media->{external}{fullpath_mediadir}\n destmedia: $builddir/media/$media->{destmedia}\n";

	       }
	}
	next;
   }
}

sub parse_copy_link {
    my ($wichaction) = @_;
    my $count;my $action;
    if ($wichaction =~ /link/) {
	$action = "ln -sf";
    } else {
	$action = "cp -av";
    }
    my $total = count_nb_to_copy($listcopy);
    print_color("$LOG copy all wanted packages to $builddir/XXXX", $color);
    # chroot packages                                                                                                                                                                
    if (!-f "$chroot_path/$isoconf->{tocopy_file}_todo_chroot") { exit 1 }
    foreach my $line (cat_("$chroot_path/$isoconf->{tocopy_file}_todo_chroot")) {
	chomp($line);
	my @list = split(/ /, $line);
	my $data = $list[$#list];
	# if (! -d $data) { print "$data is not a directory !\n"; }
	foreach my $pkg (@list) {
	    if (-f "$data/$pkg") {
		#print "$LOG $action $data/$pkg TO1 $builddir/media/main/$pkg\n";
		system("$action $data/$pkg $builddir/media/main/$pkg");
		next;
	    } elsif (-f $data) {
		my $package = basename($data);
		chomp($package);
		#print "$LOG $action $data TO2 $builddir/media/main/$package\n";
		system("$action $data $builddir/media/main/$package");
		next;                                                                                                        
	    }
	}
    }
    foreach my $tocopy (cat_($listcopy)) {
	chomp($tocopy);
	#print "info tocopy $tocopy\n";
	my @list = split(/ /, $tocopy);
	# path final
	my $data2 = $list[$#list];
	my ($path, $pkg) = $tocopy =~ m!^(/.*)/(.*)\.rpm$!;
	# une ligne avec un mot avant
        # $pkg or ($path, $pkg) = $tocopy =~ m!^\w+\s(/.*)/(.*)\.rpm$!;

	my $source_media; my $dest_media; my $package;
	if (-f "$chroot_path/$data2") {
	    my $dirn = dirname($data2);
	    ($source_media, $dest_media) = find_source_dest_media($dirn);
	    $package = basename($data2);
	    chomp($package);
	    $dest_media or print "$LOG $package is require/suggest but has no destination, removing it from list\n" and next;
	    #print "parse_copy_link package: $package, sourcemedia: $source_media, dest_media: $dest_media\n";
	    #print "parse_copy_link $LOG $action $source_media/$package TO3 $builddir/$dest_media/$package\n";
	    system("$action $source_media/$package $builddir/$dest_media/$package");
	    next;
	} elsif (!-d "$chroot_path/$data2") {
	    print "";
	    # print "$chroot_path/$data2 data2 is not a directory\n";
	} else {
	    #print "$chroot_path/$data2 data2 is a directory \n";
	    ($source_media, $dest_media) = find_source_dest_media($data2);
	    foreach my $pkg2 (@list) {
		if (-f "$chroot_path/$data2/$pkg2") {
		    #print "DATA2/pkg2 i found $chroot_path/$data2/$pkg2 \n";
		    #print "$LOG $action $source_media/$pkg2 TO4 $builddir/$dest_media/$pkg2\n";
		    system("$action $source_media/$pkg2 $builddir/$dest_media/$pkg2");
		    next;
		}
	    }
	}
	$path or next;
	if (!-d "$chroot_path/$path") {
	    print "";
	    #print "$chroot_path/$path isnt a directory\n";
	} elsif (-f "$chroot_path/$path/$pkg") {
	    #print "$chroot_path/$path/$pkg is a package\n";
	    ($source_media, $dest_media) = find_source_dest_media($path);
	    $package = basename("chroot_path/$path/$pkg");
	    chomp($package);
	    #print "$LOG $action $source_media/$package TO5 $builddir/$dest_media/$pkg\n";
	    system("$action $source_media/$package $builddir/$dest_media/$pkg");
	    next;
	} else { print "lost in space !!\n"; }
	next;
    }
}

sub rpmVersionCompare {
    my ($pkg1, $pkg2) = @_;
    my ($name_a, $av, $ar) = $pkg1 =~ /^(?:([^:]*):)?([^-]*)(?:-(.*))?$/;
    my ($name_b, $bv, $br) = $pkg2 =~ /^(?:([^:]*):)?([^-]*)(?:-(.*))?$/;
    my $rc = 0;
    if(defined($name_a) && ! defined($name_b)) {
        return 1;
    } elsif(!defined($name_a) && defined($name_b)) {
        return -1;
    } else {
        $rc = URPM::rpmvercmp($name_a, $name_b) if (defined($name_a) && defined($name_b));
        if ($rc == 0) {
            $rc = URPM::rpmvercmp($av, $bv);
            if ($rc == 0) {
                if(defined($ar) && !defined($br)) {
                    return 1;
                } elsif(!defined($ar) && defined($br)) {
                    return -1;
                } elsif (!defined($ar) &&  !defined($br)) {
                    return 0;
                } else {
                    return URPM::rpmvercmp($ar, $br);
                }
            } else {
                return $rc;
            }
        } else {
            return $rc;
        }
  }
}

sub extra_filter {
	my ($filter) = @_;
	chomp($filter);
	print_color("$LOG compare versions of $filter and keep the latest one", $color);
	use File::Glob ':glob';
	my @l; my $keeped;
	foreach my $media (@{$isoconf->{media}{list}}) {
		@l = glob("$builddir/media/$media->{destmedia}/$filter*.rpm");
		my $l = @l; $l eq "0" and last;
		foreach (@l) { 
			my $pkg = basename($_);
			# check it will be a number after the filter name
			$pkg =~ /$filter\-\d.*/ or next;
			$keeped or $keeped = $pkg;
			print "$LOG COMPARE $pkg and $keeped: \n";
			my $r = rpmVersionCompare($pkg, $keeped);
			if ($r eq "-1") {
				$keeped = $keeped;
				print "i will keep $keeped\n";
				system("rm -vf $builddir/media/$media->{destmedia}/$pkg");
			} elsif ($r eq "1") {
                                system("rm -vf $builddir/media/$media->{destmedia}/$keeped");
				$keeped = $pkg;
				print "i will keep $keeped\n";
			};
		}
	}
}


sub remove_all_excluded {
	# this functions should be available if explicitely needed, it's really too dangerous to remove packages from a list...
    print_color("$LOG remove all RPM from exclude list", $color);
    print_color("$LOG WARNING remove needed packages can break dependencies, check rpmcheck messages", $error_color);
    foreach my $media (@{$isoconf->{media}{list}}) {
        $media->{name} or next;
        foreach my $exclude (@{$media->{exclude}}) {
            $exclude->{file} or next;
            -f $exclude->{file} or print_color("$LOG i can not find the $exclude->{file}, remove this list or fix...", $error_color);
            foreach my $totest (cat_($exclude->{file})) {
		chomp($totest);
		if (! $totest) { next };
                if ($totest =~ /^\s.*/ || $totest =~ /^\#/) {
                    next;
                } else {
		    my @l = glob("$builddir/media/$media->{destmedia}/$totest*.rpm");
		    foreach my $file (@l) {
			    $file or next;
			if (-f $file) {
			       	print "$LOG $file found, removing this file\n";
				system("rm -vf $file");
				next;
			}
		    }
            	}
		next;
	    }
	    next;
	}
    }
}

sub copy_filedeps {
    print_color("$LOG copy $mediainfo_sourcedir/$isoconf->{mediainfo}{filedeps} in $builddir/$isoconf->{mediainfo}{mediainfo_dir}", $color);
    -f "$mediainfo_sourcedir/$isoconf->{mediainfo}{filedeps}" or print "$LOG missing filedeps\n";
    system("cp -vf $mediainfo_sourcedir/$isoconf->{mediainfo}{filedeps} $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
}

sub copy_rpmsrate {
    print_color("$LOG copy $mediainfo_sourcedir/$isoconf->{mediainfo}{rpmsrate} in $builddir/$isoconf->{mediainfo}{mediainfo_dir}", $color);
    -f "$mediainfo_sourcedir/$isoconf->{mediainfo}{rpmsrate}" or print "$LOG missing rpmsrate\n";
    system("cp -vf $mediainfo_sourcedir/$isoconf->{mediainfo}{rpmsrate} $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
}

sub copy_compssusers {
    print_color("$LOG copy $mediainfo_sourcedir/$isoconf->{mediainfo}{compssusers} in $builddir/$isoconf->{mediainfo}{mediainfo_dir}", $color);
    -f "$mediainfo_sourcedir/$isoconf->{mediainfo}{compssusers}" or print "$LOG missing compssusers\n";
    system("cp -vf $mediainfo_sourcedir/$isoconf->{mediainfo}{compssusers} $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
}

sub use_genhdlist2 {
    my $destmedia;
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	if ($destmedia eq $media->{destmedia}) { next }
	$destmedia = $media->{destmedia};
	print_color("$LOG build hdlists and synthesys in $builddir/media/$media->{destmedia}", $color);
	system("genhdlist2 -v --clean --file-deps $builddir/media/media_info/$isoconf->{medianfo}{filedeps} --media_info-dir $builddir/media/media_info/ $builddir/media/$media->{destmedia}");
	system("mv $builddir/media/$destmedia/media_info/hdlist.cz $builddir/media/media_info/hdlist_$media->{name}.cz");
	system("mv $builddir/media/$destmedia/media_info/synthesis.hdlist.cz $builddir/media/media_info/synthesis.hdlist_$media->{name}.cz");
#		system("genhdlist2 --quiet --clean --no-hdlist --file-deps $builddir/media/media_info/$isoconf->{medianfo}{filedeps} $builddir/media/$media->{destmedia}");
    }
    system("cd $builddir/media/media_info/ && md5sum *.cz > MD5SUM");
}

sub rpmcheck_launch {
    my $options = "-explain -failures";
    my $destmedia;
    print_color("$LOG launch rpmcheck to check hdlists", $color);
	foreach my $media (@{$isoconf->{media}{list}}) {
		$media->{name} or next;
		if ($destmedia eq $media->{destmedia}) { next }
		$destmedia = $media->{destmedia};
		system("cp $builddir/media/$destmedia/media_info/hdlist.cz $builddir/media/media_info/hdlist_$media->{name}.cz");
	}
    system("zcat $builddir/media/media_info/hdlist*.cz | rpmcheck $options");
}

sub remove_hdlist {
	my $destmedia;
	if ($isoconf->{based_on} gt "2009.1") {
		print_color("$LOG remove hdlists*.cz from $builddir/media/media_info", $color);
		system("rm -vf $builddir/media/media_info/hdlist*.cz");
		foreach my $media (@{$isoconf->{media}{list}}) {
	                $media->{name} or next;
			if ($destmedia eq $media->{destmedia}) { next }
			$destmedia = $media->{destmedia};
			print_color("$LOG remove hdlists from $builddir/media/$destmedia/media_info", $color);
			system("rm -vf $builddir/media/$destmedia/media_info/hdlist.cz");
			system("cd $builddir/media/$destmedia/media_info ; md5sum *.cz > MD5SUM");
		}
	}
}

sub copy_all_pubkey {
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	if ($media->{pubkey} eq "") {
	    print_color("$LOG no pubkey to copy (usefull when mixing main/contrib packages)", $error_color);
	} else {
	    # media/ dir will be added in the copy_pubkey functions
	    print_color("$LOG $repo/$based_on/$arch/media/$media->{mediadir}/$media->{pubkey}, $media->{destmedia}", $color);
	    copy_pubkey("$repo/$based_on/$arch/media/$media->{mediadir}/$media->{pubkey}", "$media->{destmedia}");
	}
	next;
    }
}

sub list_medias_available {
	#my @mediacfg = cat_("$repo/$based_on/$arch/$isoconf->{mediainfo_dir}/media.cfg");
	print_color("$LOG list all medias availables in $repo/$based_on/$arch repositery", $color);
	my @mediacfg = cat_("$repo/$based_on/$arch/media/media_info/media.cfg");
	my @medias = grep {/^name=/} @mediacfg;
	foreach my $media (@medias) {
		$media =~ s/name\=//; 
		$media and print("$media");
	}
}

sub check_repo_hdlist {
    my $options = "-explain -failures";
    print_color("$LOG check media repositery hdlists in $repo/$version/$arch", $color);
    system("zcat $repo/$version/$arch/media/media_info/hdlist*ain.cz | rpmcheck $options");
}

sub umount_all_mountloop {
    print "$LOG umount all mounted loop dir\n";
    foreach (@to_umount) {
	system("sudo umount $chroot_path/$_");
    }
}

sub main_media {
    my ($option) = @_;
    list_media;
    create_media;
    copy_all_pubkey;

    create_mediacfg;

    copy_filedeps;
    copy_rpmsrate;
    copy_compssusers;

    generate_list_of_rpm_2_copy;
    concatenate_files;
    target_size;
    # link or copy
    if ($option =~ /copy/) { 
	parse_copy_link("copy");
    } else {
	parse_copy_link("link");
    }

    # solve doble issue 
    if (-f $isoconf->{mediainfo}{filter}) {
	    foreach my $filtered (cat_($isoconf->{mediainfo}{filter})) {
		$filtered or next;
    		extra_filter($filtered);
	    }
    } else { print_color("$LOG $isoconf->{mediainfo}{filter} file doesnt exist", $error_color) } 

    # remove packages listed in exclude list
    remove_all_excluded;
    umount_all_mountloop;
    use_gendistrib;
#    use_genhdlist2;
    rpmcheck_launch;
    remove_hdlist;
    show_size;
}

1;

